# Copyright 2023- Le Filament (https://le-filament.com)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html)

from odoo import fields, models


class ResPartner(models.Model):
    _inherit = "res.partner"

    # ------------------------------------------------------
    # Fields declaration
    # ------------------------------------------------------
    financial_contract_guarantee_ids = fields.One2many(
        comodel_name="financial.contract.guarantee",
        inverse_name="partner_id",
        string="Contrats de garanties",
    )
    financial_contract_guarantee_count = fields.Integer(
        compute="_compute_financial_contract_guarantee_count"
    )
    guarantee_amount_ongoing = fields.Float(
        string="En cours de garantie", compute="_compute_ongoing_guarantee"
    )
    guarantee_amount_left = fields.Float(
        string="Montant de garantie disponible", compute="_compute_ongoing_guarantee"
    )
    is_amount_guarantee_limit = fields.Boolean(
        string="Plafond de garantie dépassé", compute="_compute_ongoing_guarantee"
    )

    # ------------------------------------------------------
    # SQL Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Default methods
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    def _compute_financial_contract_guarantee_count(self):
        for partner in self:
            partner.financial_contract_guarantee_count = (
                partner.financial_contract_guarantee_ids.__len__()
            )

    def _compute_ongoing_guarantee(self):
        for partner in self:
            guarantee_ids = partner.financial_contract_guarantee_ids.filtered(
                lambda guarantee: guarantee.company_id == self.env.company
                and guarantee.state == "contract"
            )
            ongoing_guarantee_amount = sum(guarantee_ids.mapped("remaining_capital"))
            partner.guarantee_amount_ongoing = ongoing_guarantee_amount
            partner.guarantee_amount_left = (
                self.env.company.amount_guarantee_limit - ongoing_guarantee_amount
            )
            partner.is_amount_guarantee_limit = (
                True
                if ongoing_guarantee_amount > self.env.company.amount_guarantee_limit
                else False
            )

    # ------------------------------------------------------
    # Onchange / Constraints
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD methods (ORM overrides)
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------
    def action_view_contract_guarantee(self):
        self.ensure_one()
        return {
            "type": "ir.actions.act_window",
            "name": "Contrats de garantie",
            "res_model": "financial.contract.guarantee",
            "view_mode": "list,form",
            "domain": [("partner_id", "=", self.id)],
            "flags": {"mode": "readonly"},
        }

    # ------------------------------------------------------
    # Business methods
    # ------------------------------------------------------
